package com.gct.edge.config;

import com.gct.edge.quartztask.DataSendTask;
import com.gct.edge.quartztask.MemoryMaintenanceTask;
import com.gct.edge.quartztask.ModbusReadTask;
import org.quartz.JobBuilder;
import org.quartz.JobDetail;
import org.quartz.Trigger;
import org.quartz.TriggerBuilder;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import static org.quartz.CronScheduleBuilder.cronSchedule;

/**
 * class name: QuartzTaskConfig
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/5/21 上午10:21
 */
@Configuration
public class QuartzTaskConfig {

    @Value("${gct.cron.modbus-read}")
    private String modbusReadCron;

    @Value("${gct.cron.memory-maintenance}")
    private String memoryMaintenanceCron;

    @Value("${gct.cron.data-send}")
    private String dataSendCron;

    @Bean
    public JobDetail jobDetail1() {
        try {
            return JobBuilder.newJob(ModbusReadTask.class)
                    .withIdentity("ModbusReadTask" ,"Group1")
                    .storeDurably()
                    .build();
        } catch (Exception e) {
            LoggerFactory.getLogger(this.getClass()).error("Create JobDetail bean failed:{},jobName->{},jobGroup->{}", e.getMessage(),"ModbusReadTask","Group1");
            throw e;
        }
    }

    @Bean
    public Trigger trigger1() {
        try {
            return TriggerBuilder.newTrigger()
                    .forJob("ModbusReadTask", "Group1")
                    .withSchedule(cronSchedule(modbusReadCron)).build();
        } catch (Exception e) {
            LoggerFactory.getLogger(this.getClass()).error("Create Trigger bean failed:{},jobName->{},jobGroup->{}", e.getMessage(),"ModbusReadTask","Group1");
            throw e;
        }
    }


    @Bean
    public JobDetail jobDetail2() {
        try {
            return JobBuilder.newJob(MemoryMaintenanceTask.class)
                    .withIdentity("MemoryMaintenanceTask" ,"Group2")
                    .storeDurably()
                    .build();
        } catch (Exception e) {
            LoggerFactory.getLogger(this.getClass()).error("Create JobDetail bean failed:{},jobName->{},jobGroup->{}", e.getMessage(),"MemoryMaintenanceTask","Group2");
            throw e;
        }
    }

    @Bean
    public Trigger trigger2() {
        try {
            return TriggerBuilder.newTrigger()
                    .forJob("MemoryMaintenanceTask", "Group2")
                    .withSchedule(cronSchedule(memoryMaintenanceCron)).build();
        } catch (Exception e) {
            LoggerFactory.getLogger(this.getClass()).error("Create Trigger bean failed:{},jobName->{},jobGroup->{}", e.getMessage(),"MemoryMaintenanceTask","Group2");
            throw e;
        }
    }



    @Bean
    public JobDetail jobDetail3() {
        try {
            return JobBuilder.newJob(DataSendTask.class)
                    .withIdentity("DataSendTask" ,"Group3")
                    .storeDurably()
                    .build();
        } catch (Exception e) {
            LoggerFactory.getLogger(this.getClass()).error("Create JobDetail bean failed:{},jobName->{},jobGroup->{}", e.getMessage(),"DataSendTask","Group3");
            throw e;
        }
    }

    @Bean
    public Trigger trigger3() {
        try {
            return TriggerBuilder.newTrigger()
                    .forJob("DataSendTask", "Group3")
                    .withSchedule(cronSchedule(dataSendCron)).build();
        } catch (Exception e) {
            LoggerFactory.getLogger(this.getClass()).error("Create Trigger bean failed:{},jobName->{},jobGroup->{}", e.getMessage(),"DataSendTask","Group3");
            throw e;
        }
    }
}
