package com.gct.edge.service.excutor;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.gct.edge.entity.DumpRecord;
import com.gct.edge.service.DumpRecordService;
import com.gct.edge.service.abst.AoidProdStateService;
import com.gct.edge.service.abst.AoidSinglegtYieldService;
import com.gct.edge.service.abst.DgnsGtcvrresultService;
import com.gct.edge.service.util.SshUtil;
import com.sun.org.apache.bcel.internal.generic.IF_ACMPEQ;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.lang.reflect.Field;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * @author xusirui 2024/4/18
 */
@Service
@Slf4j
public class DataSendService {

    @Autowired
    AoidProdStateService aoidProdStateService;

    @Autowired
    AoidSinglegtYieldService aoidSinglegtYieldService;

    @Autowired
    DgnsGtcvrresultService dgnsGtcvrresultService;

    @Autowired
    DumpRecordService dumpRecordService;

    @Value("${gct.server.host}")
    private String host;

    @Value("${gct.server.password}")
    private String password;

    @Value("${gct.server.path}")
    private String path;

    @Value("${gct.local.dump-path}")
    private String localDumpPath;

    private final static String lockStr = "lock";

    public <T> void dataSend(String tableName, String timeTagColumn, IService<T> sqlService, Class<T> clz) {
        try {
            DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
            DumpRecord lastDumpRecord = dumpRecordService.getOne(
                    new QueryWrapper<DumpRecord>().eq("table_name", tableName));
            File dataFile = new File(localDumpPath + "/" + tableName + ".txt");
            BufferedWriter bw = new BufferedWriter(new FileWriter(dataFile));
            List<T> dataList;
            String nowTimeStr = LocalDateTime.now().minusMinutes(1).format(df);
            if (Objects.isNull(lastDumpRecord)) {
                dataList = sqlService.list(new QueryWrapper<T>()
                        .last(String.format("where datetime(%s) < datetime('%s')", timeTagColumn, nowTimeStr)));
                lastDumpRecord = new DumpRecord();
                lastDumpRecord.setTableName(tableName);
                lastDumpRecord.setDumpTime(nowTimeStr);
            } else {
                dataList = sqlService.list(new QueryWrapper<T>()
                        .last(String.format("where datetime(%s) between datetime('%s') and datetime('%s')",
                                timeTagColumn, lastDumpRecord.getDumpTime(), nowTimeStr)));
                lastDumpRecord.setDumpTime(nowTimeStr);
            }
            if (dataList.isEmpty()) {
                log.info("data not found in {}, ignore send data, time: {}", tableName, lastDumpRecord.getDumpTime());
                return;
            }
            for (T t : dataList) {
                bw.write(serialRow(t, clz));
            }
            bw.close();
            SshUtil.sendLocalFile("root", password, host, 22, dataFile.getPath(), path, true);
            synchronized (lockStr){
                if (Objects.isNull(dumpRecordService.getOne(new QueryWrapper<DumpRecord>().eq("table_name", tableName))))dumpRecordService.save(lastDumpRecord);
                else dumpRecordService.update(lastDumpRecord, new QueryWrapper<DumpRecord>().eq("table_name", tableName));
            }
        } catch (Exception e) {
            log.error("send data failed, table_name:{}, message: {}", tableName, e.getMessage());
            return;
        }
        log.info("data send success,{}", tableName);
    }

    private <T> String serialRow(T t, Class<T> clz) {
        StringBuilder res = new StringBuilder();
        res.append("|");
        try {
            for (Field field : clz.getDeclaredFields()) {
                String getMethodName = "get" + field.getName().substring(0, 1).toUpperCase() + field.getName().substring(1);
                Object val = clz.getMethod(getMethodName).invoke(t);
                if (Objects.isNull(val)) res.append("|");
                else res.append(val.toString()).append("|");
            }
            res.append("\n");
        } catch (Exception e) {
            log.error("serial data failed: {}, message: {}", JSONObject.toJSONString(t), e.getMessage());
        }
        return res.toString();
    }

}
