package com.gct.edge.service.excutor;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gct.edge.entity.AoidConfigData;
import com.gct.edge.entity.AoidProdState;
import com.gct.edge.entity.CalculateInputData;
import com.gct.edge.entity.DumpRecord;
import com.gct.edge.service.DumpRecordService;
import com.gct.edge.service.abst.AoidConfigDataService;
import com.gct.edge.service.abst.AoidProdStateService;
import com.gct.edge.zdly.aoid.core.GlobalDataManager;
import com.gct.etl.ModbusDataBody;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

import com.gct.etl.ModbusRTUReader;

/**
 * @author xusirui 2024/3/7
 */
@Service
@Slf4j
public class ModbusReadService {

    @Autowired
    CalculateTaskExecutor executor;

    @Autowired
    AoidConfigDataService aoidConfigDataService;

    @Autowired
    DumpRecordService dumpRecordService;

    @Autowired
    AoidProdStateService aoidProdStateService;

    @Autowired
    GlobalDataManager globalDataManager;

    private LocalDateTime lastRefreshTime = LocalDateTime.now();

    public void readDataAndSubmitTask() {
        LocalDateTime now = LocalDateTime.now();
        if (now.minusMinutes(10).isAfter(lastRefreshTime)){
            lastRefreshTime = now;
            globalDataManager.refreshData();
        }
        List<AoidConfigData> list = aoidConfigDataService.list();
        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        if (list.isEmpty()){
            log.error("no aoid config data found,please check file conf/well-setting.txt");
        }

        for (AoidConfigData aoidConfigData : list) {
            try {
                ModbusDataBody readBody = ModbusRTUReader.readBody(Integer.parseInt(aoidConfigData.getJh()));
                List<List<Double>> sgt = readBody.card;
                Double s = readBody.s;
                Double n = readBody.n;
                String prodTime = LocalDateTime.now().format(df);
                String checkTime = readBody.prodTime;
                if (Objects.isNull(sgt) || Objects.isNull(s) || Objects.isNull(n) ||Objects.isNull(prodTime) || s < 1e-6 || n < 1e-6) {
                    log.error("read Rtu modbus failed, sgt: {},s: {},n: {},prodTime: {}", sgt, s, n, prodTime);
                    continue;
                }
                //todo：use date to check the same data and filter
                DumpRecord lastDumpRecord = dumpRecordService.getOne(
                        new QueryWrapper<DumpRecord>().eq("table_name", aoidConfigData.getJh()));
                if (Objects.isNull(lastDumpRecord)){
                    lastDumpRecord = new DumpRecord();
                    lastDumpRecord.setTableName(aoidConfigData.getJh());
                    lastDumpRecord.setDumpTime(checkTime);
                    dumpRecordService.save(lastDumpRecord);
                }else {
                    if (lastDumpRecord.getDumpTime().equals(checkTime))continue;
                    else {
                        lastDumpRecord.setDumpTime(checkTime);
                        dumpRecordService.update(lastDumpRecord,new QueryWrapper<DumpRecord>().eq("table_name", aoidConfigData.getJh()));
                    }
                }

                double[][] sgtDouble = JSONArray.toJavaObject((JSON) JSONArray.toJSON(sgt), double[][].class);
                AoidProdState aoidProdState = new AoidProdState();
                aoidProdState.setWellId(aoidConfigData.getJh());
                aoidProdState.setProdState(1);
                aoidProdState.setCreateTime(prodTime);
                aoidProdStateService.save(aoidProdState);
                executor.submit(new CalculateInputData(aoidConfigData.getJh(), LocalDateTime.parse(prodTime, df), s, n, sgtDouble));
            } catch (Exception e) {
                log.error("readDataAndSubmitTask error, detail: {}, cause: {}", e.getMessage(),e.getCause());
            }
        }


        /*
        test use
         */
//        executor.submit(new CalculateInputData("1", LocalDateTime.now(), 6.91, 1.0, SGTUtil.encodeToUseArray("0,9570,0,9668,1,9738,3,9765,4,9796,7,9802,9,9767,12,9740,16,9705,19,9671,24,9698,28,9750,34,9819,39,9965,45,10113,52,10250,59,10408,66,10558,74,10739,82,10908,91,10964,99,10901,108,10694,117,10452,125,10230,134,10150,143,10184,151,10349,160,10562,168,10706,177,10782,186,10728,194,10578,202,10422,212,10272,220,10223,229,10283,238,10395,246,10544,254,10646,263,10674,272,10630,281,10522,289,10401,298,10323,307,10298,315,10335,324,10428,333,10509,341,10581,350,10602,358,10565,367,10507,376,10432,384,10385,393,10379,401,10421,409,10480,419,10549,428,10584,436,10591,445,10570,453,10526,462,10482,471,10452,479,10449,488,10464,497,10490,505,10522,514,10542,523,10564,531,10568,540,10553,548,10535,557,10509,566,10482,574,10458,583,10446,592,10441,600,10446,609,10462,617,10480,625,10493,632,10495,639,10500,646,10494,652,10478,657,10472,662,10461,667,10451,672,10442,675,10447,679,10446,682,10454,684,10464,687,10461,688,10448,690,10430,691,10391,691,10325,691,10256,691,10184,690,10107,688,10050,687,10019,684,10011,682,10016,679,10046,675,10083,672,10089,667,10058,662,10014,657,9919,652,9773,646,9682,639,9564,632,9416,625,9257,617,9112,609,9025,601,9081,593,9249,586,9471,578,9660,570,9748,562,9709,555,9559,547,9374,539,9197,532,9133,524,9189,516,9294,509,9473,501,9581,493,9631,484,9556,478,9439,470,9295,462,9183,455,9182,447,9215,438,9334,430,9452,424,9523,416,9553,408,9480,401,9403,393,9291,385,9227,378,9239,370,9271,362,9392,355,9460,347,9532,339,9526,331,9474,324,9420,316,9324,308,9312,301,9307,293,9362,285,9439,277,9483,270,9532,262,9497,254,9462,247,9418,239,9356,231,9339,224,9339,216,9321,208,9348,200,9400,193,9412,185,9435,177,9476,170,9468,162,9436,154,9437,146,9401,139,9348,131,9341,123,9340,115,9320,108,9348,100,9391,92,9402,85,9424,77,9459,69,9445,62,9424,54,9424,46,9400,38,9358,31,9367,23,9368,15,9357,8,9392,0,9458,0,9503")));
    }
}
