package com.gct.edge.zdly.aoid.core;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gct.common.util.SGTUtil;
import com.gct.edge.entity.*;
import com.gct.edge.mapper.AoidSinglewellConfigMapper;
import com.gct.edge.service.abst.*;
import com.gct.edge.zdly.aoid.entity.ProductionCoreData;
import com.gct.edge.zdly.dgns.core.DiagnoseVisitor;
import com.sun.org.apache.bcel.internal.generic.IF_ACMPEQ;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.SQLOutput;
import java.time.LocalDateTime;
import java.util.*;

/**
 * @author xusirui 2024/3/20
 */
@Service
@Slf4j
public class CalculateProductionService {

    @Autowired
    GlobalDataManager globalDataManager;

    @Autowired
    AoidRodsModuleService aoidRodsModuleService;

    @Autowired
    AoidConfigDataService aoidConfigDataService;

    @Autowired
    AoidGkModelService aoidGkModelService;

    @Autowired
    AoidSinglewellConfigMapper aoidSinglewellConfigMapper;

    @Autowired
    AoidSinglegtYieldService aoidSinglegtYieldService;

    private LocalDateTime lastRefreshTime = LocalDateTime.now();

    /**
     * calculate production begin function
     */
    public void calculateProductionEntrance(DiagnoseVisitor diagnoseVisitor){
        ProductionDataManager productionDataManager = new ProductionDataManager();
        dataPrepareJob(diagnoseVisitor,productionDataManager);
        calculateProductionJob(productionDataManager);
        dataPersistJob(productionDataManager);
    }


    private void dataPrepareJob(DiagnoseVisitor diagnoseVisitor,ProductionDataManager productionDataManager){
        productionDataManager.dgnsGtbaseparam = diagnoseVisitor.gtbaseparam;
        productionDataManager.dgnsGtcvrresult = diagnoseVisitor.cvResult;
        List<AoidRodsModule> aoidRodsModules = aoidRodsModuleService.list();
        if (aoidRodsModules.isEmpty()){
            log.error("lacking rods modules");
        }
        productionDataManager.aoidRodsModule = aoidRodsModules.get(0);
        productionDataManager.productionCoreData = new ProductionCoreData(globalDataManager);
        productionDataManager.rev = productionDataManager.productionCoreData.new GtRevision();
        productionDataManager.calbgt = productionDataManager.productionCoreData.new calbgt(productionDataManager.aoidRodsModule);
        productionDataManager.pc = productionDataManager.productionCoreData.new pumpCard();
        productionDataManager.aoidConfigData = aoidConfigDataService.getOne(new QueryWrapper<AoidConfigData>().eq("jh", productionDataManager.dgnsGtbaseparam.getWellname()));
        productionDataManager.gkModel = aoidGkModelService.getOne(new QueryWrapper<AoidGkModel>().eq("result", productionDataManager.dgnsGtcvrresult.getResult2()));
        productionDataManager.singlewellConfig = aoidSinglewellConfigMapper.getOne("1");
    }

    private void calculateProductionJob(ProductionDataManager productionDataManager){
        DgnsGtbaseparam dgnsGtbaseparam = productionDataManager.dgnsGtbaseparam;
        DgnsGtcvrresult dgnsGtcvrresult = productionDataManager.dgnsGtcvrresult;
        ProductionCoreData.GtRevision rev = productionDataManager.rev;
        ProductionCoreData.calbgt calbgt = productionDataManager.calbgt;
        ProductionCoreData.pumpCard pc = productionDataManager.pc;
        ProductionCoreData productionCoreData = productionDataManager.productionCoreData;
        AoidConfigData aoidConfigData = productionDataManager.aoidConfigData;
        AoidSinglewellConfig singlewellConfig = productionDataManager.singlewellConfig;
        AoidGkModel gkModel = productionDataManager.gkModel;
        if (Objects.isNull(dgnsGtcvrresult)){
            log.error("无诊断结果已跳过量液");
            return;
        }
        rev.sgt_encode(dgnsGtbaseparam);
        calbgt.cal_bgt(rev, dgnsGtcvrresult.getResult2());

        if (Objects.isNull(aoidConfigData)){
            log.error("没有配置井筒基础数据,已跳过量液");
        }
        calbgt.c_pump_diagram(dgnsGtbaseparam, pc, globalDataManager.getGlobalData("oil-tube-inner-size",Integer.class),
                (Double) globalDataManager.getGlobalData("oil-tube-inner-size-loss",List.class).get(0)
                ,Double.parseDouble(aoidConfigData.getBs()),Double.parseDouble(aoidConfigData.getBj()));
        productionDataManager.liqResultList = productionCoreData.cal_ylandon(SGTUtil.encodeToUseList(dgnsGtbaseparam.getSgt()), pc.getDbu(), Double.parseDouble(dgnsGtbaseparam.getS()),
                Double.parseDouble(dgnsGtbaseparam.getN()), dgnsGtcvrresult.getResult2(), Double.parseDouble(aoidConfigData.getBj()),
                Double.parseDouble(aoidConfigData.getBs()), Double.parseDouble(aoidConfigData.getHsl()), Double.parseDouble(aoidConfigData.getSmd()),
                Double.parseDouble(aoidConfigData.getYmd()), Double.parseDouble(aoidConfigData.getQyb()), Double.parseDouble(aoidConfigData.getDym()),
                Double.parseDouble(gkModel.getGkxs()), singlewellConfig.getGradient1(), singlewellConfig.getGradient10(),
                singlewellConfig.getGradient2(), singlewellConfig.getGradient3(), singlewellConfig.getGradient4(), singlewellConfig.getGradient5(),
                singlewellConfig.getGradient6(), singlewellConfig.getGradient7(), singlewellConfig.getGradient8(), singlewellConfig.getGradient9(),
                singlewellConfig.getDividenum1(), singlewellConfig.getDividenum2(), singlewellConfig.getDividenum3(), singlewellConfig.getDividenum4(),
                singlewellConfig.getDividenum5(), singlewellConfig.getDividenum6(), singlewellConfig.getDividenum7(),
                singlewellConfig.getDividenum8(), singlewellConfig.getDividenum9(), singlewellConfig.getDividenum10());
        if (productionDataManager.liqResultList.isEmpty()){
            log.error("量液结果计算出错");
        }

    }

    private void dataPersistJob(ProductionDataManager productionDataManager){
        DgnsGtbaseparam dgnsGtbaseparam = productionDataManager.dgnsGtbaseparam;
        DgnsGtcvrresult dgnsGtcvrresult = productionDataManager.dgnsGtcvrresult;
        AoidConfigData aoidConfigData = productionDataManager.aoidConfigData;
        List<Double> liqResultList = productionDataManager.liqResultList;
        AoidSinglegtYield singlegtYield = new AoidSinglegtYield();
        singlegtYield.setWellCommonName(dgnsGtcvrresult.getWellname());
        singlegtYield.setWellId(dgnsGtcvrresult.getWellname());
        singlegtYield.setRefrencewellId(dgnsGtbaseparam.getId());
        singlegtYield.setRefrencecjsj(dgnsGtbaseparam.getRecivetime());
        singlegtYield.setRefrencesgt(dgnsGtbaseparam.getSgt());
        singlegtYield.setQyb(aoidConfigData.getQyb());
        singlegtYield.setHsl(aoidConfigData.getHsl());
        singlegtYield.setBj(aoidConfigData.getBj());
        singlegtYield.setSgt(dgnsGtbaseparam.getSgt());
        singlegtYield.setBgt(SGTUtil.encodeToString(productionDataManager.calbgt.bgt
                .stream().map(subList -> subList.stream().mapToDouble(Double::doubleValue).toArray()).toArray(double[][]::new)));
        singlegtYield.setBdxs(productionDataManager.gkModel.getGkxs());
        singlegtYield.setBs(aoidConfigData.getBs());
        Double cmd = Double.parseDouble(aoidConfigData.getBs()) - Double.parseDouble(aoidConfigData.getDym());
        if (cmd<=0)cmd=1d;
        singlegtYield.setCmd(cmd.toString());
        singlegtYield.setEczdjg(dgnsGtcvrresult.getResult2());
        singlegtYield.setFrequence(dgnsGtbaseparam.getN());
        singlegtYield.setStroke(dgnsGtbaseparam.getS());
        singlegtYield.setYy(aoidConfigData.getYy());
        singlegtYield.setTy(aoidConfigData.getTy());
        singlegtYield.setHy(singlegtYield.getHy());
        singlegtYield.setProdDate(dgnsGtbaseparam.getRecivetime());
        singlegtYield.setRefrencesgt(dgnsGtbaseparam.getSgt());
        singlegtYield.setZdjg(dgnsGtcvrresult.getResult2());
        singlegtYield.setScsj("24");
        singlegtYield.setPumpdepth(aoidConfigData.getBs());
        singlegtYield.setYzzj(aoidConfigData.getBj());
        singlegtYield.setLiqProdDaily(liqResultList.get(0).toString());
        singlegtYield.setOilProdDaily(liqResultList.get(1).toString());
        singlegtYield.setWaterProdDaily(liqResultList.get(2).toString());
        singlegtYield.setGasProdDaily(liqResultList.get(3).toString());
        singlegtYield.setBx(liqResultList.get(4).toString());
        singlegtYield.setLlpl(liqResultList.get(5).toString());
        singlegtYield.setSjcc(liqResultList.get(6).toString());
        singlegtYield.setYxcc(liqResultList.get(7).toString());
        singlegtYield.setZcc(liqResultList.get(8).toString());
        singlegtYield.setYcc(liqResultList.get(9).toString());
        singlegtYield.setCcss(liqResultList.get(10).toString());
        singlegtYield.setLsss(liqResultList.get(11).toString());
        singlegtYield.setGyss(liqResultList.get(12).toString());
        singlegtYield.setStrokeRatio(liqResultList.get(13).toString());

        aoidSinglegtYieldService.save(singlegtYield);
        log.info("功图日产液量: " + singlegtYield.getLiqProdDaily());
    }

}
