package com.gct.edge.zdly.dgns.algo;

import com.gct.edge.entity.DgnsGtbaseparam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @Author: Jyx
 * @Since: 2021/1/25 17:23
 */
public class DiagramValidate {

    private static final int gtPointsNumConditionlimit = 50;

    /**
     * 功图预识别（参数，拐点，面积等计算）
     *
     * @param gt 功图数据
     */
    public void analysisParam(double[][] gt, DgnsGtbaseparam gtbaseparam) {
        gtbaseparam.setSgtpointnum(gt.length);
        Double sMax = Double.MIN_VALUE;
        Integer sMaxP = 0;
        Double lMax = Double.MIN_VALUE;
        Integer lMaxP = 0;
        Double sMin = Double.MAX_VALUE;
        Integer sMinP = 0;
        Double lMin = Double.MAX_VALUE;
        Integer lMinP = 0;
        Double area = 0d;
        Double upload = 0d;
        Double download = 0d;

        if (gt.length > 0 && gtbaseparam.getSgtpointnum() > gtPointsNumConditionlimit) {
            for (int i = 0; i < gtbaseparam.getSgtpointnum(); i++) {
                if (gt[i][0] > sMax) {
                    sMax = gt[i][0];
                    sMaxP = i;
                }
                if (gt[i][1] > lMax) {
                    lMax = gt[i][1];
                    lMaxP = i;
                }
                if (gt[i][0] < sMin) {
                   sMin =gt[i][0];
                    sMinP = i;
                }
                if (gt[i][1] < lMin) {
                   lMin = gt[i][1];
                   lMinP = 0;
                }

                //上下行对半分
                if (i < gtbaseparam.getSgtpointnum() / 2 - 1 && i < gtbaseparam.getSgtpointnum() - 1) {
                    double h = Math.abs(gt[i][0] - gt[i + 1][0]);
                    double Si = (gt[i][1] + gt[i + 1][1]) * h / 2.0;
                    area +=Si;
                    upload += gt[i][1] / (gtbaseparam.getSgtpointnum() / 2 - 1);
                } else if (i >= gtbaseparam.getSgtpointnum() / 2 - 1 && i < gtbaseparam.getSgtpointnum() - 1) {
                    double h = Math.abs(gt[i][0] - gt[i + 1][0]);
                    double Si = (gt[i][1] + gt[i + 1][1]) * h / 2.0;
                    area-=Si;
                    download += gt[i][1] / (gtbaseparam.getSgtpointnum() / 2 + 1);
                }
                //---

            }
            gtbaseparam.setSMax(Double.toString(sMax));
            gtbaseparam.setSMin(Double.toString(sMin));
            gtbaseparam.setSMaxPointLocation(sMaxP);
            gtbaseparam.setSMinPointLocation(sMinP);
            gtbaseparam.setLMax(Double.toString(lMax));
            gtbaseparam.setLMin(Double.toString(lMin));
            gtbaseparam.setLMaxPointLocation(lMaxP);
            gtbaseparam.setLMinPointLocation(lMinP);
            gtbaseparam.setS(Double.toString(sMax-sMin));
            gtbaseparam.setL(Double.toString(lMax-lMin));
            gtbaseparam.setArea(Double.toString(area));
            gtbaseparam.setUpload(Double.toString(upload));
            gtbaseparam.setDownload(Double.toString(download));

            Double sAtSlopeMinLoc = Double.MAX_VALUE;
            Double sAtSlopeMaxLoc = Double.MIN_VALUE;
            Double slopeDownloadMin = Double.MAX_VALUE;
            Double slopeDownloadMax = Double.MIN_VALUE;

            for (int i = 0; i < gtbaseparam.getSgtpointnum(); i++) {
                // 根据功图获取有效冲程
                // 算法:取下半段功图的斜率最大值跟最小值对应的点的冲程差
                if (i >= gtbaseparam.getSgtpointnum() / 2 - 1 && i < gtbaseparam.getSgtpointnum() - 1) {

                    double xl = Math.abs((gt[i][1] - gt[i + 1][1]) / (gt[i][0] - gt[i + 1][0] + 0.000001));//这里取的是个固定值，可能会有问题
                    if (xl < slopeDownloadMin) {
                        slopeDownloadMin = xl;
                        sAtSlopeMinLoc = gt[i][0];
                    }
                    if (xl > slopeDownloadMax) {
                        slopeDownloadMax = xl;
                        sAtSlopeMaxLoc = gt[i][0];
                    }
                }
            }
            if (sAtSlopeMaxLoc!=Double.MIN_VALUE && sAtSlopeMinLoc!=Double.MAX_VALUE)
                        gtbaseparam.setSEffect(Double.toString(Math.abs(sAtSlopeMaxLoc - sAtSlopeMinLoc)));
            else gtbaseparam.setSEffect("0");

            gtbaseparam.setLMid(Double.toString((lMax+lMin) / 2));
            try {
                gtbaseparam.setLFistdownturnpoint(Double.toString(getFirstTurningPoint2(gt, gtbaseparam.getSgtpointnum())));
            } catch (Exception exception) {
                gtbaseparam.setLFistdownturnpoint(null);
                Logger logger = LoggerFactory.getLogger(this.getClass());
                logger.error("loadFistDownTurnPoint cal ERROR:{}", exception.getMessage());
            }
        }
        Map<String, Object> resMap = new ProblemA()
                .findProACardL(gt, Double.valueOf(gtbaseparam.getL()),Double.valueOf(gtbaseparam.getS()), Double.valueOf(gtbaseparam.getN()));
        gtbaseparam.setIslegal(Integer.parseInt(resMap.get("result").toString()));
        gtbaseparam.setRemark(resMap.get("remark").toString());
    }


    /**
     * @param gt         计算功图第一拐点载荷判断 供液不足间歇出液
     * @param gtPointNum 功图像素点数
     */
    public double getFirstTurningPoint2(double[][] gt, int gtPointNum) {
        int start = gtPointNum / 2;
        double minL = gt[start][1];
        double maxL = gt[start][1];
        double mins = gt[start][0];
        double maxs = gt[start][0];
        List<double[]> res = new ArrayList<>();
        res.add(gt[start]); //range [start,gtpointNum-1]
        for (int i = start + 1; i < gtPointNum; i++) {
            res.add(gt[i]);
            if (gt[i][1] < minL) minL = gt[i][1];
            if (gt[i][1] > maxL) maxL = gt[i][1];
            if (gt[i][0] > maxs) maxs = gt[i][0];
            if (gt[i][0] < mins) mins = gt[i][0];
        }
        double Ldd = maxL - minL;
        double sdd = maxs - mins;
        int countT = (int) Math.ceil(Math.sqrt(Math.pow(Ldd / 0.01, 2) + Math.pow(sdd / 0.1, 2)));
        double[][] gtBezierTemp = BezierInterpolation(res.toArray(new double[][]{}), countT, 2);
        for (int i = 0; i < gtBezierTemp.length; i++) {
            double zh = gtBezierTemp[i][1];
            double s = gtBezierTemp[i][0];
            for (int j = i + 1; j < gtBezierTemp.length; j++) {
                if (gtBezierTemp[j][0] < s) {
                    if (Math.abs(zh - gtBezierTemp[j][1]) < 0.1)
                        if (Math.abs(zh - gtBezierTemp[j][1]) < Math.abs(s - gtBezierTemp[j][0]) * 0.1)
                            return zh;
                }
            }
        }

        return 0.0;
    }

    /**
     * 贝塞尔插值fuc
     * 终将会存在硬件限制，不能真正的无限阶，但是计算功图200阶小case
     *
     * @param poss      贝塞尔曲线控制点坐标（包含起点终点）
     * @param precision 精度，需要计算的该条贝塞尔曲线上的点的数目
     * @param dimersion 该条贝塞尔曲线上的点维度，坐标轴数（二维坐标，三维坐标...）
     */
    public double[][] BezierInterpolation(double[][] poss, int precision, int dimersion) {
        //double[][] bezier_points = new double[0][];
        List<double[]> bezier_points = new ArrayList<>();
        //维度，坐标轴数（二维坐标，三维坐标...）
        if (dimersion != 2) return bezier_points.toArray(new double[][]{});
        //贝塞尔曲线控制点数（阶数），此说法不太对，但是便于理解记忆
        int number = poss.length;
        //控制点数不小于 2 ，至少为二维坐标系
        if (number < 2 || dimersion < 2)
            return bezier_points.toArray(new double[][]{});

        //double[][] result = new double[precision][];
        //计算杨辉三角
        double[] mi = new double[number];
        mi[0] = mi[1] = 1;
        for (int i = 3; i <= number; i++) {
            double[] t = new double[(i - 1)];
            System.arraycopy(mi, 0, t, 0, i - 1);
            /*for (int j = 0; j < i - 1; j++) {
                t[j] = mi[j];
            }*/
            mi[0] = mi[i - 1] = 1;
            for (int j = 0; j < i - 2; j++) {
                mi[j + 1] = t[j] + t[j + 1];
            }
        }

        //计算坐标点
        for (int i = 0; i < precision; i++) {
            double t = (double) i / precision;
            double[] temp = new double[dimersion];
            temp[0] = temp[1] = 0;
            for (int j = 0; j < dimersion; j++) {
                for (int k = 0; k < number; k++) {
                    temp[j] += Math.pow(1 - t, number - k - 1) * poss[k][j] * Math.pow(t, k) * mi[k];
                    //幂运算改为循环乘，防止溢出???
                    //var c_temp=poss[k][j]* mi[k];
                    //for (var c = 1; c <= number - k - 1; c++)
                    //    c_temp *= (1 - t);
                    //for (var c = 1; c <= k; c++)
                    //    c_temp *= t;
                    //temp += c_temp;
                }
            }
            bezier_points.add(temp);
        }
        return bezier_points.toArray(new double[][]{});
    }
}