package com.gct.edge.zdly.dgns.core;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gct.edge.zdly.dgns.algo.CalGtSimilar;
import com.gct.common.util.SGTUtil;

import com.gct.edge.zdly.dgns.thriftclient.GTComputerViewRecognitionClient;
import com.gct.edge.zdly.dgns.thriftclient.ThriftClientResponseData;
import com.gct.edge.entity.CalculateInputData;
import com.gct.edge.entity.DgnsGtmloutcode;
import com.gct.edge.entity.DgnsGtstandard;
import com.gct.edge.service.abst.DgnsGtmloutcodeService;
import com.gct.edge.service.abst.DgnsGtstandardService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Objects;

/**
 * @author xusirui 2021/6/29
 */
@Slf4j
@Component
public class FirstDiagnose implements DiagnosePart {

    @Autowired
    GTComputerViewRecognitionClient gtComputerViewRecognitionClient;

    @Autowired
    DgnsGtstandardService dgnsGtstandardService;

    @Autowired
    DgnsGtmloutcodeService dgnsGtmloutcodeService;

    public static String DEFAULT_TAG = "一次诊断无法识别";

    @Override
    public void accept(DiagnoseVisitor diagnoseVisitor) {
        diagnoseVisitor.cvResult.setResult1("-1"); //-1代表一次诊断不成功，默认为-1,如果成功会被修改为对应工况
        //转换数据
        CalculateInputData inputData = diagnoseVisitor.calculateInputData;
        List<List<Double>> sgtList = SGTUtil.encodeToUseList(diagnoseVisitor.gtbaseparam.getSgt());

        if (diagnoseVisitor.gtbaseparam.getIslegal() == 0) {//功图不合法 不进行一次诊断 ,todo: 是否需要标准功图？？？？
            diagnoseVisitor.appendLog("一次诊断：功图不合法,"+ diagnoseVisitor.gtbaseparam.getRemark());
        } else {
            //调用一次诊断
            ThriftClientResponseData<Integer> integerThriftClientResponseData
                    = gtComputerViewRecognitionClient.tryGTPredictType(sgtList);
            if (!integerThriftClientResponseData.isSuccess()) {
                diagnoseVisitor.appendLog("一次诊断：网络故障，无法执行一次诊断");
                log.error("Thrift调用失败");
            } else {
                diagnoseVisitor.gtmloutcode = dgnsGtmloutcodeService.getOne(new QueryWrapper<DgnsGtmloutcode>()
                        .eq("mlcode", integerThriftClientResponseData.getData()));
                diagnoseVisitor.cvResult.setResult1(Objects.isNull(diagnoseVisitor.gtmloutcode)?DEFAULT_TAG: diagnoseVisitor.gtmloutcode.getGkdec());
                diagnoseVisitor.appendLog("一次诊断：" + (Objects.isNull(diagnoseVisitor.gtmloutcode) ? DEFAULT_TAG : diagnoseVisitor.gtmloutcode.getGkdec()));
            }
        }
        //获取该井生产日期最近 7 天内出现次数最多的功图作为标准功图
        List<DgnsGtstandard> dgnsGtstandardList = dgnsGtstandardService.getAllStandGt(inputData.getWellId());
        if (dgnsGtstandardList == null || dgnsGtstandardList.size() == 0) {
            diagnoseVisitor.appendLog("一次诊断：缺失标准功图");
        } else {
            Double maxSimilar = 0.0;
            DgnsGtstandard maxSimilarDgns = dgnsGtstandardList.get(0);
            for (DgnsGtstandard dgnsGtstandard : dgnsGtstandardList) {
                Double similar = CalGtSimilar.cal2(CalGtSimilar.encodeToList(dgnsGtstandard.getFuture()),sgtList);
                if (similar > maxSimilar) {
                    maxSimilar = similar;
                    maxSimilarDgns = dgnsGtstandard;
                }
            }
            diagnoseVisitor.standardGt = maxSimilarDgns;
            diagnoseVisitor.similar = maxSimilar;
            dgnsGtstandardService.calStdGt(diagnoseVisitor.gtbaseparam, diagnoseVisitor.cvResult.getResult1());
            diagnoseVisitor.appendLog("标准功图解释："+maxSimilarDgns.getGraphicfeatureinterpretation()+"，相似度："+Math.floor(maxSimilar *   100)+"%");
        }
        dgnsGtstandardService.calStdGt(diagnoseVisitor.gtbaseparam,diagnoseVisitor.cvResult.getResult1());
    }
}
