package com.gct.edge.zdly.dgns.core;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gct.edge.zdly.dgns.decisiontree.DecisionTree;
import com.gct.edge.zdly.dgns.decisiontree.DecisionTreeGenerator;
import com.gct.edge.zdly.dgns.decisiontree.DependentDataFieldEnum;
import com.gct.edge.entity.CalculateInputData;
import com.gct.edge.entity.DgnsGtcvrparam;
import com.gct.edge.entity.DgnsGtgktype;
import com.gct.edge.entity.DgnsGtmloutcode;
import com.gct.edge.service.abst.DgnsGtcvrparamService;
import com.gct.edge.service.abst.DgnsGtgktypeService;
import com.gct.edge.service.abst.DgnsGtmloutcodeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.naming.OperationNotSupportedException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
/**
 * @author xusirui 2021/6/29
 */
@Slf4j
@Component
public class SecondDiagnose implements DiagnosePart {

    @Autowired
    DgnsGtmloutcodeService dgnsGtmloutcodeService;

    @Autowired
    DgnsGtgktypeService dgnsGtgktypeService;

    @Autowired
    DgnsGtcvrparamService dgnsGtcvrparamService;

    @Autowired
    DecisionTreeGenerator decisionTreeGenerator;

    @Override
    public void accept(DiagnoseVisitor diagnoseVisitor) {
        //决策树初始化，页面需求数据加载
        DecisionTreeGenerator generator = initDecisionTree(diagnoseVisitor);
        DgnsGtcvrparam gtcvrparam = getCvAlgorithm(diagnoseVisitor);
        if (!checkValid(diagnoseVisitor)) return;
        if (Objects.isNull(gtcvrparam)) return;
        DecisionTree tree = generator.generateDecisionTreeFromJson(gtcvrparam.getParamcontent());
        try {
            Object result2 = generator.runDecisionTree(tree);
            if (!Objects.isNull(result2)) diagnoseVisitor.cvResult.setResult2(result2.toString());
        } catch (OperationNotSupportedException | IllegalStateException e) {
            diagnoseVisitor.appendLog("二次诊断：操作不支持或数据不存在");
        }
    }

    //检查前面运行的状态，不合法则不继续执行
    private boolean checkValid(DiagnoseVisitor diagnoseVisitor) {
        boolean flag = true;
        if (diagnoseVisitor.gtbaseparam.getIslegal() == 0) {  //一厂 准东 需求差异 通过数据库配置区分
            DgnsGtmloutcode gtmloutcode = dgnsGtmloutcodeService.getOne(new QueryWrapper<DgnsGtmloutcode>()
                    .eq("gkdec", diagnoseVisitor.gtbaseparam.getRemark())
                    .or().eq("gkcode::varchar", diagnoseVisitor.gtbaseparam.getRemark()));
            diagnoseVisitor.cvResult.setResult2(diagnoseVisitor.gtbaseparam.getRemark()); //set default
            if (!Objects.isNull(gtmloutcode) && !Objects.isNull(gtmloutcode.getGkcode())) {
                DgnsGtgktype gtgktype = dgnsGtgktypeService.getOne(new QueryWrapper<DgnsGtgktype>()
                        .eq("id", gtmloutcode.getGkcode()));
                if (!Objects.isNull(gtgktype)) {
                    diagnoseVisitor.cvResult.setResult2(gtgktype.getGkname());
                }
            }
            flag = false;
        }
        String result1 = diagnoseVisitor.cvResult.getResult1();
        if (Objects.isNull(result1) || result1.equals(FirstDiagnose.DEFAULT_TAG)) {
            diagnoseVisitor.appendLog("二次诊断：一次诊断未成功进行,不执行二次诊断");
            flag = false;
        }
        return flag;
    }

    //生成决策树默认参数值
    private DecisionTreeGenerator initDecisionTree(DiagnoseVisitor diagnoseVisitor) {
        try {
            CalculateInputData diagnoseMsg = diagnoseVisitor.calculateInputData;
            Map<String, Object> initMap = new HashMap<>();
            initMap.put(DependentDataFieldEnum.JH.getEnDec(), diagnoseMsg.getWellId());
            initMap.put("orgId", 0);
            initMap.put("curGtTime", diagnoseMsg.getProdDate());
            initMap.put(DependentDataFieldEnum.BZSJ.getEnDec(), diagnoseVisitor.standardGt.getSgtprodate());
            initMap.put(DependentDataFieldEnum.BZGK.getEnDec(), diagnoseVisitor.standardGt.getGraphicfeatureinterpretation());
            initMap.put(DependentDataFieldEnum.XSD.getEnDec(), diagnoseVisitor.similar);
            initMap.put(DependentDataFieldEnum.YCZDJG.getEnDec(), Objects.isNull(diagnoseVisitor.gtmloutcode)?null: diagnoseVisitor.gtmloutcode.getGkdec());
            initMap.put("dgnsGtbaseparam", diagnoseVisitor.gtbaseparam);
            initMap.put("sgt", diagnoseVisitor.gtbaseparam.getSgt());
            diagnoseVisitor.initMap = initMap;
            decisionTreeGenerator.setArgs(diagnoseVisitor.initMap);
            diagnoseVisitor.globeData = decisionTreeGenerator.getGlobeData();
        } catch (NoSuchFieldException e) {
            diagnoseVisitor.appendLog("二次诊断：运行参数不存在");
        } catch (IllegalAccessException e) {
            diagnoseVisitor.appendLog("二次诊断：非法操作");
        }
        return decisionTreeGenerator;
    }

    //获取二次诊断算法
    private DgnsGtcvrparam getCvAlgorithm(DiagnoseVisitor diagnoseVisitor) {
        //todo: 按照井区所属org 查找算法
        DgnsGtcvrparam gtcvrparam = null;
        try {
            List<DgnsGtcvrparam> gtcvrparams = null;
            if (Objects.nonNull(diagnoseVisitor.gtmloutcode)) {
                gtcvrparams = dgnsGtcvrparamService.list(new QueryWrapper<DgnsGtcvrparam>()
                        .eq("paramid", diagnoseVisitor.gtmloutcode.getGkcode()));
            }
            if (Objects.nonNull(gtcvrparams) && !gtcvrparams.isEmpty()) {
                if (Objects.isNull(gtcvrparam))gtcvrparam = gtcvrparams.get(0);
            }
        } catch (Exception e) {
            e.printStackTrace();
            diagnoseVisitor.appendLog("二次诊断：诊断算法不唯一");
        }
        if (Objects.isNull(diagnoseVisitor.gtmloutcode)) {
            diagnoseVisitor.appendLog("二次诊断：找不到一次诊断对应工况");
            diagnoseVisitor.cvResult.setResult2("二次诊断失败");
        } else {
            //设置二次诊断默认结果，防止二次诊断失败时无结果输出
            DgnsGtgktype gtgktype = dgnsGtgktypeService.getOne(new QueryWrapper<DgnsGtgktype>()
                    .eq("id", diagnoseVisitor.gtmloutcode.getGkcode()));
            if (Objects.isNull(gtgktype)) diagnoseVisitor.cvResult.setResult2(diagnoseVisitor.gtmloutcode.getGkdec());
            else diagnoseVisitor.cvResult.setResult2(gtgktype.getGkname());
        }
        if (Objects.isNull(gtcvrparam)) {
            diagnoseVisitor.appendLog("二次诊断：找不到该工况的诊断算法");
        }
        return gtcvrparam;
    }
}
