package com.gct.edge.zdly.dgns.core;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gct.common.util.StringUtil;
import com.gct.edge.entity.DgnsGtmloutcode;
import com.gct.edge.service.abst.DgnsGtmloutcodeService;
import com.gct.edge.zdly.dgns.decisiontree.DecisionTree;
import com.gct.edge.zdly.dgns.decisiontree.DecisionTreeGenerator;
import com.gct.edge.entity.DgnsGtcvrThird;
import com.gct.edge.entity.DgnsGtcvrparam;
import com.gct.edge.service.abst.DgnsGtcvrThirdService;
import com.gct.edge.service.abst.DgnsGtcvrparamService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.naming.OperationNotSupportedException;
import java.util.*;
import java.util.stream.Collectors;


/**
 * @author xusirui 2022/2/15
 */
@Component
@Slf4j
public class ThirdDiagnose implements DiagnosePart {

    @Value("${gct.diagnose.thirdDiagenose.enabled:#{false}}")
    boolean enabled;

    @Autowired
    DecisionTreeGenerator decisionTreeGenerator;

    @Autowired
    DgnsGtcvrparamService dgnsGtcvrparamService;

    @Autowired
    DgnsGtcvrThirdService dgnsGtcvrThirdService;

    @Autowired
    DgnsGtmloutcodeService dgnsGtmloutcodeService;

    @Override
    public void accept(DiagnoseVisitor diagnoseVisitor) {
        if (!enabled) return;
        List<String> algoNames = getThirdAlgoName(diagnoseVisitor.cvResult.getResult1());
        if (algoNames.isEmpty()){
            diagnoseVisitor.appendLog("未配置三次诊断算法，不执行三次诊断");
            return;
        }
        if (Objects.isNull(diagnoseVisitor.standardGt.getId())){
            diagnoseVisitor.appendLog("缺失标准功图，不执行三次诊断");
            return;
        }
        Map<String, DgnsGtcvrparam> nameParamMap = getCvParamsByAlgoNames(algoNames);
        //execute third dgns
        for(String x:algoNames){
            DgnsGtcvrparam gtcvrparam = nameParamMap.get(x);
            if (Objects.isNull(gtcvrparam)){
                diagnoseVisitor.appendLog("缺失三次诊断算法:"+x);
            }else {
                DecisionTree tree = decisionTreeGenerator.generateDecisionTreeFromJson(gtcvrparam.getParamcontent());
                try {
                    Object result3 = decisionTreeGenerator.runDecisionTree(tree);
                    if (Objects.nonNull(result3) && !result3.toString().equals("#")){
                        diagnoseVisitor.cvResult.setResult2(result3.toString());
                        break;
                    }
                } catch (OperationNotSupportedException e) {
                    log.error("三次诊断算法执行出错: cause by{}",e.getMessage());
                    diagnoseVisitor.appendLog("三次诊断算法执行出错:"+x);
                }
            }
        }
    }

    private Map<String, DgnsGtcvrparam> getCvParamsByAlgoNames(List<String> algoNames){
        List<DgnsGtcvrparam> list = dgnsGtcvrparamService
                .list(new QueryWrapper<DgnsGtcvrparam>().in("paramitemname", algoNames).eq("paramid", 0));
        Map<String, DgnsGtcvrparam> map = new HashMap<String, DgnsGtcvrparam>();
        if (!Objects.isNull(list)&&!list.isEmpty()){
            list.forEach(x->{
                map.put(x.getParamitemname(),x);
            });
        }
        return map;
    }

    private List<String> getThirdAlgoName(String result1) {
        List<String> algoNames = new ArrayList<>();
        Integer gkcode = dgnsGtmloutcodeService.getOne(new QueryWrapper<DgnsGtmloutcode>().eq("gkdec", result1)).getGkcode();

        List<DgnsGtcvrThird> list = dgnsGtcvrThirdService.list(
                new QueryWrapper<DgnsGtcvrThird>().eq("result1", gkcode)
        );
        if (list.isEmpty()) return algoNames;
        DgnsGtcvrThird dgnsGtcvrThird = list.get(0);
        if (Objects.isNull(dgnsGtcvrThird) || StringUtil.isBlankOrEmpty(dgnsGtcvrThird.getAlgoNameOrderly()))return algoNames;
        try{
            algoNames = Arrays.stream(dgnsGtcvrThird.getAlgoNameOrderly().split(";")).collect(Collectors.toList());
        }catch (Exception e){
            log.error("获取三次诊断算法失败，cause by {}",e.getMessage());
        }
        return algoNames;
    }


}
