package com.gct.edge.zdly.dgns.decisiontree;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.proxy.CallbackFilter;
import org.springframework.cglib.proxy.Enhancer;
import org.springframework.stereotype.Component;

import javax.naming.OperationNotSupportedException;
import java.lang.annotation.*;
import java.lang.reflect.Method;
import java.util.concurrent.ConcurrentHashMap;

/**
 * class name: DecisionTreeInterpreter
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/1/25 下午3:41
 */
@Component
public class ComputeTemplateProxyFactory {
    private final static Enhancer en = new Enhancer();

    private final static ConcurrentHashMap<Object, IComputeTemplate> computeTemplates = new ConcurrentHashMap<>();

    private ComputeInvokeMethodInterceptor interceptor;

    @Autowired
    public ComputeTemplateProxyFactory(ComputeInvokeMethodInterceptor interceptor) {
        this.interceptor = interceptor;
    }

    /**
     * 适用于有入参的 且有返回参的
     *
     * @param target
     * @return
     */
    public IComputeTemplate getProxy(Object target) {
        if (computeTemplates.containsKey(target))
            return computeTemplates.get(target);

        synchronized (en){
            en.setSuperclass(target.getClass());
            en.setCallbackFilter(new ComputeInvokeMethodFilter());
            en.setCallback(this.interceptor);
            IComputeTemplate template = (IComputeTemplate) en.create();
            computeTemplates.put(target, template);
            return template;
        }

    }

    /**
     * interface name: IComputeTemplate
     *
     * @author lloyd
     * @version 1.0
     * @since 2021/1/23 下午2:47
     */
    public interface IComputeTemplate {

        Object[] invokeCompute(Object[] args) throws OperationNotSupportedException;

        //ArrayList<Object> invokeCompute(Object[] args) throws OperationNotSupportedException;
    }

    @Retention(RetentionPolicy.RUNTIME)
    @Documented
    @Target(ElementType.TYPE)
    @Component
    public @interface ComputeTemplate {
        String name() default "";

        String describe() default "";

        String prefix() default "";//""gct.compute.templates.";

        /*

        TODO:生成可读模板必要属性----后面可视化的时候再做
        如:
         !(@in)
         @in +  @in
         @in || @in

         and so on..
         */
    }


    class ComputeInvokeMethodFilter implements CallbackFilter {
        @Override
        public int accept(Method method) {
            if (method.getName().equals("invokeCompute")) return 0;
            else
                return 0;//此处留作回调扩展 方法时使用
        }
    }
}


