package com.gct.edge.zdly.dgns.decisiontree;

import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.lang.annotation.*;
import java.util.HashMap;
import java.util.Map;

/**
 * class name: DependentDataConstructor
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/1/23 下午2:47
 */
@Component
@Slf4j
public class DependentDataConstructor {

    private static final ThreadLocal<Map<String, Object>> globeData = new ThreadLocal<>();

//    private BaseDataClient baseDataClient;

    public DependentDataConstructor() {
    }

/*    public DependentDataConstructor(BaseDataClient baseDataClient) {

        this.baseDataClient = baseDataClient;
    }*/

    public void defaultLoadGlobeData(Map<String, Object> initMap) {
        globeData.set(new HashMap<>());
        Map<String, Object> data = globeData.get();
        initMap.forEach((x, y) -> data.put(x, y));
    }


    //key is not load values Description
    private void extendLoadGlobeData(String key) {
        DependentDataFieldEnum fieldEnum;
        try {
            fieldEnum = DependentDataFieldEnum.valueOf(key);
        } catch (Exception e) {
            log.error("can not found dataLoad of key {}", key);
            return;
        }
        IDependentDataLoad dataLoad = TemplateConfiguration.getDataLoadTemplateBy(fieldEnum.getTemp());
        if (dataLoad != null) dataLoad.loadData(globeData.get());
    }

    @Retention(RetentionPolicy.RUNTIME)
    @Documented
    @Target(ElementType.TYPE)
    @Component
    public @interface DependentDataLoad {
        String Name() default "";
    }

    public interface IDependentDataLoad {
        void loadData(Map<String, Object> gdMap);
    }


    public Object getGlobeDataValue(String key) {
        Map<String, Object> data = globeData.get();
        if (!data.containsKey(key)) {
            extendLoadGlobeData(key);
        }
        return data.get(key);
    }

    public void putGlobeDataValue(String key, Object val) {
        Map<String, Object> data = globeData.get();
        if (data.containsKey(key)) data.replace(key, val);
        else data.put(key, val);// to replace or insert new value
    }
}


