package com.gct.edge.zdly.dgns.decisiontree;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanInstantiationException;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * class name: SpringBeansAwareComputeTemplate
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/1/25 下午1:15
 */
@Component
@Slf4j
public class TemplateConfiguration implements ApplicationContextAware {

    private static ApplicationContext applicationContext;
    private static Map<String, Object> computeTemplateBeans;
    private static Map<String, Object> dataLoadTemplateBeans;

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        TemplateConfiguration.applicationContext = applicationContext;

        //注册bean时就能放到自己的hashmap中
        Map<String, Object> computerBeans = TemplateConfiguration.applicationContext.getBeansWithAnnotation(ComputeTemplateProxyFactory.ComputeTemplate.class);
        if (computerBeans.isEmpty()) return;

        computeTemplateBeans = new HashMap<>();

        for (Object val : computerBeans.values()) {
            ComputeTemplateProxyFactory.ComputeTemplate an = val.getClass().getAnnotation(ComputeTemplateProxyFactory.ComputeTemplate.class);
            if (computeTemplateBeans.containsKey(an.prefix() + an.name())) {
                log.error("repeat registry the compute bean by  prefix:{},name:{}", an.prefix(), an.name());
                throw new BeanInstantiationException(val.getClass(), String.format("repeat registry the compute bean by  prefix:{%s},name:{%s}", an.prefix(), an.name()));
            }
            computeTemplateBeans.put(an.prefix() + an.name(), val);
        }

        dataLoadTemplateBeans = new HashMap<>();

        Map<String, Object> dataLoadBeans = TemplateConfiguration.applicationContext.getBeansWithAnnotation(DependentDataConstructor.DependentDataLoad.class);
        for(Object val : dataLoadBeans.values()){
            DependentDataConstructor.DependentDataLoad an = val.getClass().getAnnotation(DependentDataConstructor.DependentDataLoad.class);
            if (dataLoadBeans.containsKey(an.Name())){
                log.error("repeat registry the compute bean by name:{}", an.Name());
                throw new BeanInstantiationException(val.getClass(), String.format("repeat registry the compute bean by name:{}", an.Name()));
            }
            dataLoadTemplateBeans.put(an.Name(),val);
        }

    }

    //获取applicationContext
    public static ApplicationContext getApplicationContext() {
        return applicationContext;
    }

    public static Map<String, Object> getComputeTemplateBeans() {
        return computeTemplateBeans;
    }

    //通过name获取 Bean.
    public static Object getBean(String name) {
        return applicationContext.getBean(name);
    }

    //通过class获取Bean.
    public static <T> T getBean(Class<T> clazz) {
        return applicationContext.getBean(clazz);
    }

    //通过name,以及Clazz返回指定的Bean
    public static <T> T getBean(String name, Class<T> clazz) {
        return applicationContext.getBean(name, clazz);
    }

    public static ComputeTemplateProxyFactory.IComputeTemplate getComputeTemplateBy(String name, String prefix) {
        if (name == null || name.isEmpty()) {
            return null;
        }
        if (computeTemplateBeans == null) {
            log.error("not init computeTemplateBeans");
            return null;
        }
        if (!computeTemplateBeans.containsKey(name)) {
            log.error("not found your required computeTemplateBean by prefix:{},name:{}", prefix, name);
            return null;
        }
        return (ComputeTemplateProxyFactory.IComputeTemplate) computeTemplateBeans.get(prefix + name);
    }

    public static DependentDataConstructor.IDependentDataLoad getDataLoadTemplateBy(String name) {
        if (name == null) {
            return null;
        }
        if (dataLoadTemplateBeans == null) {
            log.error("not init dataLoadTemplateBeans");
            return null;
        }
        if (!dataLoadTemplateBeans.containsKey(name)) {
            log.error("not found your required dataLoadTemplateBeans by name:{}", name);
            return null;
        }
        return (DependentDataConstructor.IDependentDataLoad) dataLoadTemplateBeans.get(name);
    }


}

