package com.gct.edge.zdly.dgns.thriftclient;


import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;

/**
 * <p>
 * ThriftClient: for diagram_card recognition
 * <p>
 * use @Autowired to inject your dependence location
 * </p>
 *
 * @author lloyd
 * @version 1.0
 * @since 2021.1.21
 */
@Slf4j
//@Component
public class GTComputerViewRecognitionClientBak {
    private static final int DEFAULTPREDICTTYPE = -1;
    private static final double DEFAUSIMILAR = -1;

    private final ThriftSocketClientPool pool;

    @Autowired
    public GTComputerViewRecognitionClientBak(ThriftSocketClientPool pool) {
        this.pool = pool;
    }

    private int GTPredictType(List<List<Double>> sgt, int sgtPointNum) {
        gtcvrService.Client client = null;
        int result = DEFAULTPREDICTTYPE;

        try {
            client = pool.getThriftClient();
            result = client.GTPredictType(sgt, sgtPointNum);
        } catch (Exception e) {
            log.warn("RPC:ERROR:GTPredictType");
            e.printStackTrace();
        } finally {
            if (client != null)
                pool.releaseThriftClient(client);
        }
        return result;
    }

    private double SGTSimilarByGrayVal(List<List<Double>> baseSGT, int baseSGTPointNum, List<List<Double>> matchSGT, int matchSGTPointNum) {

        gtcvrService.Client client = null;
        double result = DEFAUSIMILAR;

        try {
            client = pool.getThriftClient();
            result = client.SGTSimillarByGrayVal(baseSGT, baseSGTPointNum, matchSGT, matchSGTPointNum);
        } catch (Exception e) {
            log.warn("RPC:ERROR:SGTSimilarByGrayVal");
            e.printStackTrace();
        } finally {
            if (client != null)
                pool.releaseThriftClient(client);
        }
        return result;
    }

    /**
     * 进行功图图形识别
     *
     * @param sgt         功图
     * @param sgtPointNum 功图点数
     * @return ResponseData
     */
    public ThriftClientResponseData<Integer> tryGTPredictType(List<List<Double>> sgt, int sgtPointNum) {
        int failedCount = 0;
        do{
            int pType = GTPredictType(sgt, sgtPointNum);
            if (pType == DEFAULTPREDICTTYPE) {
                try{
                    Thread.sleep(50);
                }catch (Exception e){
                    e.printStackTrace();
                }
            }else {
                return new ThriftClientResponseData<Integer>().setData(pType).setSuccess(true);
            }
        }while (failedCount++<5);
        return new ThriftClientResponseData<Integer>().setData(null).setSuccess(false);
    }

    /**
     * 灰度计算两幅功图的特征相似度
     *
     * @param baseSGT          功图
     * @param baseSGTPointNum  功图点数
     * @param matchSGT         功图
     * @param matchSGTPointNum 功图点数
     * @return ResponseData
     */
    public ThriftClientResponseData<Double> trySGTSimilarByGrayVal(List<List<Double>> baseSGT, int baseSGTPointNum, List<List<Double>> matchSGT, int matchSGTPointNum) {
        double pSimilar = SGTSimilarByGrayVal(baseSGT, baseSGTPointNum, matchSGT, matchSGTPointNum);
        if (pSimilar == DEFAULTPREDICTTYPE)
            return new ThriftClientResponseData<Double>().setData(null).setSuccess(false);
        else return new ThriftClientResponseData<Double>().setData(pSimilar).setSuccess(true);
    }

}
