package com.gct.edge.zdly.dgns.thriftclient;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * Configuration for ThriftClient
 * <p>
 * thrift client,connect C++ thrift server
 *
 * </p>
 *
 * @author lloyd
 * @version 2.0
 * @since 2021.1.21
 */
@Configuration
@EnableConfigurationProperties({ThriftClientProperties.class, ThriftSocketClientPoolProperties.class})
public class ThriftClientConfig {

    private final ThriftClientProperties thriftClientProperties;
    private final ThriftSocketClientPoolProperties thriftSocketClientPoolProperties;

    @Autowired
    public ThriftClientConfig(ThriftClientProperties thriftClientProperties, ThriftSocketClientPoolProperties thriftSocketClientPoolProperties) {
        this.thriftClientProperties = thriftClientProperties;
        this.thriftSocketClientPoolProperties = thriftSocketClientPoolProperties;
    }

    /**
     * use common-pool for thrift client
     * @return ThriftSocketClientPool
     */
    @Bean
    public ThriftSocketClientPool getThriftSocketClientPool() {
        //这里一定要加这个设置，不然springboot启动会报已经注册了某个bean的错误
        thriftSocketClientPoolProperties.setJmxEnabled(false);
        ThriftSocketClientFactory thriftSocketClientFactory=new ThriftSocketClientFactory(thriftClientProperties);
        return new ThriftSocketClientPool(thriftSocketClientFactory, thriftSocketClientPoolProperties);
    }

    @Bean(destroyMethod = "destroy")
    public ThriftClientFactory thriftClientFactory(){
        return new ThriftClientFactory(thriftClientProperties);
    }

}
