package com.gct.edge.zdly.dgns.thriftclient;

import lombok.extern.slf4j.Slf4j;
import org.apache.thrift.TException;
import org.apache.thrift.protocol.TBinaryProtocol;
import org.apache.thrift.transport.TSocket;
import org.apache.thrift.transport.TTransport;
import org.apache.thrift.transport.TTransportException;
import org.springframework.beans.factory.DisposableBean;

/**
 * class name: ThriftClientFactory
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/6/30 下午5:23
 */
@Slf4j
public class ThriftClientFactory implements DisposableBean {

    private gtcvrService.Client client;
    private ThriftClientProperties properties;

    /**
     * @param properties
     */
    public ThriftClientFactory(ThriftClientProperties properties) {
        this.properties = properties;
    }

    /**
     * @return
     * @throws ValidateFailureClient
     */
    public gtcvrService.Client getClient() throws ValidateFailureClient {
        if (!validateClient()) destroy();
        else return client;

        if (client == null) {
            try {
                TTransport transport = new TSocket(properties.getHost(), properties.getPort(), properties.getTimeout());
                transport.open();
                client = new gtcvrService.Client(new TBinaryProtocol(transport));
            } catch (TTransportException e) {
//                e.printStackTrace();
                log.info("Thrift-gtcvr-rpc:connect failed==>host:{},port:{}", properties.getHost(), properties.getPort());

            }
        }
        if (!validateClient()) {
            destroy();
            throw new ValidateFailureClient("validated failure:thrift client");
        } else {
            log.info("Thrift-gtcvr-rpc:Connected==>host:{},port:{}", properties.getHost(), properties.getPort());
        }
        return client;
    }

    private boolean validateClient() {
        if (client == null) return false;
        try {
            if (client.HealthyCheck())
                return true;
            else return false;
        } catch (TException e) {
            return false;
        }
    }

    @Override
    public void destroy() {
        if (client != null)
            try {
                client.getInputProtocol().getTransport().flush();
                client.getInputProtocol().getTransport().close();

            } catch (Exception ignore) {
            } finally {
                client = null;
            }
    }
}

class ValidateFailureClient extends RuntimeException {

    public ValidateFailureClient(String message) {
        super(message);
    }
}
