package com.gct.edge.zdly.dgns.thriftclient;

import lombok.SneakyThrows;
import org.apache.commons.pool2.BasePooledObjectFactory;
import org.apache.commons.pool2.PooledObject;
import org.apache.commons.pool2.impl.DefaultPooledObject;
import org.apache.thrift.protocol.TBinaryProtocol;
import org.apache.thrift.protocol.TProtocol;
import org.apache.thrift.transport.TSocket;
import org.apache.thrift.transport.TTransport;

/**
 * @author lloyd
 * @version 1.0
 * @since 2021/1/22 下午1:53
 */
public class ThriftSocketClientFactory extends BasePooledObjectFactory<TTransport> {

    private final ThriftClientProperties thriftClientProperties;

    public ThriftSocketClientFactory(ThriftClientProperties thriftClientProperties) {
        this.thriftClientProperties = thriftClientProperties;
    }

    @Override
    public TTransport create() throws Exception {
        TTransport transport = new TSocket(thriftClientProperties.getHost(), thriftClientProperties.getPort(), thriftClientProperties.getTimeout());
//        if (!transport.isOpen()) transport.open();
        return transport;
    }

    @SneakyThrows
    @Override
    public boolean validateObject(PooledObject<TTransport> pooledObject) {
        TTransport tTransport = pooledObject.getObject();

        boolean ping = false;
        try {
            tTransport.open();
            TProtocol tProtocol = new TBinaryProtocol(tTransport);
            gtcvrService.Client client = new gtcvrService.Client(tProtocol);
            //检查TTransport是否可用
            ping = client.HealthyCheck();
        } catch (Exception ignored) {

        } finally {
//            tTransport.close();
        }
        return ping;
    }

    @Override
    public PooledObject<TTransport> wrap(TTransport tTransport) {
        return new DefaultPooledObject<>(tTransport);
    }

    @Override
    public synchronized void destroyObject(PooledObject<TTransport> pooledObject) {
        //这里的作用是保证客户端在关闭时，两边都不会报错
        if (pooledObject.getObject().isOpen()) {
            try {
                TTransport object = pooledObject.getObject();
            } catch (Exception ignore) {
            }
        }
    }

}
