package com.gct.etl;

import com.alibaba.fastjson.JSON;
import com.gct.etl.loader.DefaultConfigLoader;
import com.gct.etl.loader.DefaultMultiConfigLoader;
import com.gct.etl.loader.MultiModbusConfigLoader;
import com.ghgande.j2mod.modbus.Modbus;
import com.ghgande.j2mod.modbus.ModbusException;
import com.ghgande.j2mod.modbus.facade.ModbusSerialMaster;
import com.ghgande.j2mod.modbus.facade.ModbusTCPMaster;
import com.ghgande.j2mod.modbus.procimg.Register;
import com.ghgande.j2mod.modbus.procimg.SimpleRegister;
import com.ghgande.j2mod.modbus.util.SerialParameters;
import lombok.extern.slf4j.Slf4j;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * dec:
 * <p>
 * ModbusRTUReader
 * </p>
 *
 * @author lloyd
 * @version 1.0.1
 * @since 2024/4/8 上午10:49
 */
@Slf4j
public class ModbusRTUReader {

    static int unit_id = 1;

    //region test
    public static void main(String[] args) {
//        tcp modbus 连接测试
//        testTCP();
//        手动调试串口数据
//        debugSerial();
//        测试功图读取
        writeSample(true);//true 只刷新功图时间
        testReadDataBody();
//        testReadMultiBody();
    }

    private static void testReadMultiBody() {
        //try catch 否则出错会终止工作线程 看情况使用
        try {
            System.out.println(readBody(1));
            System.out.println(readBody(2));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void debugSerial() {
        Scanner scanner = new Scanner(System.in);
        while (true) {
            System.out.print("input serial name:");
            String serial = scanner.nextLine();
            if (serial.isEmpty()) serial = "serial";
            System.out.print("input baud rate(default 9600):");
            String br = scanner.nextLine();
            int baudRate = Integer.parseInt(br.isEmpty() ? "0" : br);
            if (baudRate <= 0) baudRate = 9600;
            System.out.print("input use 485 mod (default no,yes/no,y/n?):");
            String mode = scanner.nextLine();
            boolean isRs485 = "yes".equals(mode) || "y".equals(mode);
            System.out.print("input station id 1~9(default 1):");
            String line = scanner.nextLine();
            int stationId = Integer.parseInt(line.isEmpty() ? "1" : line);
            if (stationId <= 0 || stationId >= 10) stationId = 1;
            try {
                testSerial(serial, baudRate, isRs485, stationId);
            } catch (Exception e) {
                e.printStackTrace();
            }
            System.out.print("next? (none as exit):");
            String next = scanner.nextLine();
            if (next.isEmpty()) break;
        }
        System.out.println("exit");
        scanner.close();
    }

    public static void testTCP() throws Exception {
        ModbusTCPMaster master = new ModbusTCPMaster("192.168.31.252", 502);
        master.connect();
        master.setTimeout(200000);
        Register[] registers = master.readMultipleRegisters(1, 111, 16);
        for (Register register : registers) {
            System.out.print(register.toString() + " ");
        }
    }

    public static void testSerial(String serialName, int baudRate, boolean isRs485, int stationId) throws Exception {
        System.out.printf("serial: %s, baudRate: %d, stationId: %d, is485mode: %s%n", serialName, baudRate, stationId, isRs485);
        SerialParameters parameters = new SerialParameters();
//        parameters.setPortName("COM3");
        parameters.setPortName(serialName);
        parameters.setOpenDelay(1000);
//        parameters.setBaudRate(115200);
//        parameters.setBaudRate(9600);
        parameters.setBaudRate(baudRate);
        parameters.setRs485Mode(isRs485);
        parameters.setEncoding(Modbus.SERIAL_ENCODING_RTU);
        ModbusSerialMaster master = new ModbusSerialMaster(parameters);
        master.connect();

        Register[] registers = master.readMultipleRegisters(stationId, 115, 16);
        for (Register register : registers) System.out.print(register.toString() + " ");
        master.disconnect();
    }

    public static void testReadDataBody() {
        regConfig.serial.serialName = "ttyUSB0";//盒子串口为：ttyAMA3
        regConfig.serial.serialName = "COM5";
//        regConfig.serial.serialName = "ttyAMA3";
        regConfig.serial.baudRate = 9600;//232口使用115200,485口注释此行
        ModbusDataBody body = readBody();
        System.out.println(body);
    }

    public static void writeSample(boolean onlyRefreshTime) {
        regConfig.serial.serialName = "COM5";
        regConfig.serial.baudRate = 9600;

        connect();

        ModbusSerialMaster master = masterCache.get(regConfig.serial);
        SimpleRegister s = new SimpleRegister(4513);
        SimpleRegister n = new SimpleRegister(261);
        String jsonArr = "[[0.004,48.3],[0.006,48.31],[0.008,48.57],[0.012,49.08],[0.019,49.98],[0.031,51.05],[0.047,52.3],[0.067,53.81],[0.09,55.28],[0.115,56.78],[0.14,58.17],[0.168,59.25],[0.197,60.21],[0.229,61.21],[0.264,62.08],[0.3,62.99],[0.338,63.83],[0.377,64.58],[0.418,65.37],[0.46,66.04],[0.504,66.6],[0.551,67.18],[0.602,67.83],[0.655,68.51],[0.711,69.26],[0.77,70.06],[0.831,70.9],[0.893,71.73],[0.955,72.41],[1.017,73.01],[1.077,73.59],[1.135,74.2],[1.193,74.82],[1.252,75.52],[1.312,76.35],[1.375,77.17],[1.442,77.96],[1.512,78.54],[1.584,78.93],[1.656,78.91],[1.728,78.66],[1.798,78.15],[1.87,77.54],[1.94,76.89],[2.013,76.34],[2.086,75.88],[2.158,75.61],[2.231,75.54],[2.303,75.72],[2.375,76.1],[2.445,76.64],[2.516,77.26],[2.587,77.69],[2.659,78.03],[2.73,78.26],[2.801,78.34],[2.871,78.15],[2.943,77.85],[3.014,77.31],[3.083,76.68],[3.152,76.15],[3.22,75.78],[3.288,75.56],[3.353,75.57],[3.42,75.73],[3.486,75.76],[3.549,75.38],[3.61,75.35],[3.671,75.43],[3.729,75.58],[3.786,76.01],[3.84,76.93],[3.893,77.56],[3.943,78.0],[3.992,78.26],[4.037,78.3],[4.081,78.18],[4.128,77.88],[4.171,77.57],[4.209,77.05],[4.244,76.46],[4.277,76.01],[4.305,75.78],[4.329,75.66],[4.356,75.84],[4.385,76.22],[4.411,76.68],[4.434,77.14],[4.457,77.29],[4.474,76.96],[4.484,76.46],[4.491,75.77],[4.497,75.09],[4.501,74.75],[4.503,74.85],[4.505,75.03],[4.505,75.11],[4.505,74.71],[4.501,73.79],[4.496,72.61],[4.491,71.21],[4.483,69.69],[4.471,68.29],[4.459,67.07],[4.443,65.78],[4.422,64.46],[4.402,63.15],[4.38,61.79],[4.358,60.43],[4.337,59.15],[4.318,58.04],[4.299,57.09],[4.281,56.2],[4.26,55.35],[4.237,54.51],[4.21,53.5],[4.182,52.46],[4.153,51.58],[4.123,50.83],[4.09,50.13],[4.056,49.47],[4.02,48.79],[3.984,48.02],[3.947,47.44],[3.909,47.25],[3.871,47.41],[3.834,47.83],[3.795,48.57],[3.754,49.24],[3.714,49.75],[3.671,50.18],[3.626,50.45],[3.58,50.35],[3.534,50.0],[3.485,49.38],[3.436,48.55],[3.388,47.73],[3.338,47.17],[3.289,46.81],[3.239,46.6],[3.189,46.73],[3.135,47.09],[3.081,47.64],[3.026,48.35],[2.969,49.1],[2.911,49.57],[2.854,49.76],[2.796,49.74],[2.735,49.46],[2.673,48.98],[2.611,48.38],[2.546,47.75],[2.483,47.08],[2.422,46.58],[2.361,46.42],[2.297,46.53],[2.231,46.82],[2.163,47.28],[2.091,47.79],[2.029,48.16],[1.968,48.58],[1.906,48.94],[1.842,49.1],[1.78,49.07],[1.708,48.9],[1.637,48.54],[1.569,48.12],[1.502,47.72],[1.432,47.39],[1.358,47.14],[1.282,47.05],[1.203,47.13],[1.127,47.36],[1.055,47.73],[0.988,48.2],[0.924,48.64],[0.863,49.03],[0.799,49.37],[0.734,49.55],[0.672,49.58],[0.609,49.49],[0.55,49.27],[0.494,48.97],[0.443,48.78],[0.393,48.64],[0.345,48.58],[0.3,48.66],[0.258,48.86],[0.221,49.11],[0.187,49.47],[0.157,49.93],[0.131,50.3],[0.105,50.57],[0.081,50.73],[0.058,50.71],[0.039,50.41],[0.023,50.13],[0.013,49.65],[0.006,49.1],[0.004,48.6]]";
        Double[][] arr = JSON.parseObject(jsonArr, Double[][].class);
        SimpleRegister[] payload = new SimpleRegister[arr.length];
        SimpleRegister[] offset = new SimpleRegister[arr.length];
        for (int i = 0; i < arr.length; i++) {
            payload[i] = new SimpleRegister(Double.valueOf(arr[i][1] * 100).intValue());
            offset[i] = new SimpleRegister(Double.valueOf(arr[i][0] * 1000).intValue());
        }
        LocalDateTime now = LocalDateTime.now();
        SimpleRegister[] time = new SimpleRegister[]{new SimpleRegister(now.getYear()-2000),
                new SimpleRegister(now.getMonthValue()),
                new SimpleRegister(now.getDayOfMonth()),
                new SimpleRegister(now.getHour()),
                new SimpleRegister(now.getMinute()),
                new SimpleRegister(now.getSecond())};

        try {
            master.writeMultipleRegisters(regConfig.prodTimeReg.stationId, regConfig.prodTimeReg.register, time);
            if(onlyRefreshTime) return;
            master.writeMultipleRegisters(regConfig.sReg.stationId, regConfig.sReg.register, new Register[]{s});
            master.writeMultipleRegisters(regConfig.nReg.stationId, regConfig.nReg.register, new Register[]{n});
            master.writeMultipleRegisters(regConfig.pointNumReg.stationId, regConfig.pointNumReg.register, new Register[]{new SimpleRegister(arr.length)});
            writeArrayBatch(master, payload, regConfig.payloadRegs);
            writeArrayBatch(master, offset, regConfig.offsetRegs);

        } catch (ModbusException e) {
            throw new RuntimeException(e);
        }finally {
            System.out.println("模拟功图运行完成");
        }

    }

    private static void writeArrayBatch(ModbusSerialMaster master, SimpleRegister[] payload, List<ModbusReadConfig.Section> payloadRegs) throws ModbusException {
        for (int i = 0; i < payloadRegs.size(); i++) {
            ModbusReadConfig.Section reg = payloadRegs.get(i);
            master.writeMultipleRegisters(reg.stationId, reg.register,
                    Arrays.stream(payload).skip((i)*reg.length).limit(reg.length).collect(Collectors.toList()).toArray(new SimpleRegister[]{}));
        }
    }
    //endregion

    static MultiModbusConfigLoader configLoader = new DefaultMultiConfigLoader();
    static ModbusReadConfig regConfig = new DefaultConfigLoader().load();
    static Map<ModbusReadConfig.SerialConfig, ModbusSerialMaster> masterCache = new HashMap<>();

    static {
        //connect();
    }

    private static void connect() {
        System.out.printf("serial: %s, baudRate: %d, is485mode: %s%n", regConfig.serial.serialName, regConfig.serial.baudRate, regConfig.serial.isRs485);
        ModbusSerialMaster master = masterCache.get(regConfig.serial);
        if (Objects.nonNull(master) && master.isConnected()) return;

        SerialParameters parameters = new SerialParameters();
        parameters.setPortName(regConfig.serial.serialName);
        parameters.setOpenDelay(regConfig.serial.openDaly);
        parameters.setBaudRate(regConfig.serial.baudRate);
        parameters.setRs485Mode(regConfig.serial.isRs485);
        parameters.setEncoding(Modbus.SERIAL_ENCODING_RTU);
        master = new ModbusSerialMaster(parameters);
        try {
            master.connect();
        } catch (Exception e) {
            log.error("connect serial port failed, detail: {}", e.getMessage());
        }
        masterCache.put(regConfig.serial, master);
    }

    private static Register[] tryReadMultipleRegisters(int stationId, int register, int length) {
        try {
            return masterCache.get(regConfig.serial).readMultipleRegisters(stationId, register, length);
        } catch (ModbusException e) {
            throw new RuntimeException(e);
        }
    }

    private static List<Double> readDoubleArr(List<ModbusReadConfig.Section> regs, RegisterParser<Double> parser, int unitLen) {
        List<Double> arr = new ArrayList<>();
        for (ModbusReadConfig.Section reg : regs) {
            Register[] registers = tryReadMultipleRegisters(reg.stationId, reg.register, reg.length);
            for (int i = 0; i < registers.length; i++) {
                Register[] unit = new Register[unitLen];
                for (int j = 0; j < unitLen; j++) {
                    if (i + j >= registers.length)
                        throw new RuntimeException("register数组长度不足以支持该数据单元长度:" + unitLen + ",index:" + i + ",total:" + registers.length);
                    unit[j] = registers[i + j];
                }
                i = i + unitLen - 1;
                arr.add(parser.apply(unit));
            }
        }
        return arr;
    }

    private static Double readDoubleSingle(ModbusReadConfig.Section reg, RegisterParser<Double> parser) {
        Register[] registers = tryReadMultipleRegisters(reg.stationId, reg.register, reg.length);
        return parser.apply(registers);
    }

    private static Integer readIntegerSingle(ModbusReadConfig.Section reg, RegisterParser<Integer> parser) {
        Register[] registers = tryReadMultipleRegisters(reg.stationId, reg.register, reg.length);
        return parser.apply(registers);
    }

    public static List<List<Double>> readDiagramCard() {
        List<Double> payloadArr = readDoubleArr(regConfig.payloadRegs, regConfig.payloadParser, 1);
        List<Double> offsetArr = readDoubleArr(regConfig.offsetRegs, regConfig.offsetParser, 1);
        int size = Math.min(payloadArr.size(), offsetArr.size());
        List<List<Double>> dcArr = new ArrayList<>();
        for (int i = 0; i < size; i++) dcArr.add(Arrays.asList(offsetArr.get(i), payloadArr.get(i)));
        return dcArr;
    }

    private static String readTime(ModbusReadConfig.Section reg, RegisterParser<String> parser) {
        Register[] registers = tryReadMultipleRegisters(reg.stationId, reg.register, reg.length);
        return parser.apply(registers);
    }

    public static synchronized ModbusDataBody readBody() {
        connect();

        ModbusDataBody body = new ModbusDataBody();
        body.card = readDiagramCard();
        body.s = readDoubleSingle(regConfig.sReg, regConfig.sParser);
        body.n = readDoubleSingle(regConfig.nReg, regConfig.nParser);
        body.pointNum = readIntegerSingle(regConfig.pointNumReg, regConfig.pointNumParser);
        body.prodTime = readTime(regConfig.prodTimeReg, regConfig.dateTimeParser);
        return body;
    }

    public static synchronized ModbusDataBody readBody(int stationId) {
        regConfig = configLoader.loadById(stationId);
        return readBody();
    }
}
