package com.gct.etl;

import com.gct.etl.parser.*;

import java.util.Arrays;
import java.util.List;

/**
 * dec:
 * <p>
 * ModbusReadConfig
 * </p>
 *
 * @author lloyd
 * @version 1.0.1
 * @since 2024/4/12 上午10:17
 */
public class ModbusReadConfig {

    public SerialConfig serial;

    public static class SerialConfig {
        public String serialName = "serial";
        public int baudRate = 9600;

        public int openDaly = 1000;
        public boolean isRs485 = false;
    }

    public List<Section> payloadRegs = Arrays.asList(
            new Section(1, 1749, 50),
            new Section(1, 1799, 50),
            new Section(1, 1849, 50),
            new Section(1, 1899, 50));
    public List<Section> offsetRegs = Arrays.asList(
            new Section(1, 1499, 50),
            new Section(1, 1549, 50),
            new Section(1, 1599, 50),
            new Section(1, 1649, 50));
    public Section nReg = new Section(1, 1488, 1);
    public Section sReg = new Section(1, 1489, 1);
    public Section prodTimeReg = new Section(1, 1481, 6);
    public Section pointNumReg = new Section(1, 1487, 1);


    public RegisterParser<Double> payloadParser = new PayloadToDouble();
    public RegisterParser<Double> offsetParser = new OffsetToDouble();
    public RegisterParser<Double> nParser = new NToDouble();
    public RegisterParser<Double> sParser = new SToDouble();
    public RegisterParser<Integer> pointNumParser = new DefaultParser();
    public RegisterParser<String> dateTimeParser = new DateTimeParser();

    public ModbusReadConfig() {
    }

    public ModbusReadConfig setSerial(SerialConfig serial) {
        this.serial = serial;
        return this;
    }

    public static class Section {
        public int stationId;
        public int register;
        public int length;

        public Section(int stationId, int register, int length) {
            this.stationId = stationId;
            this.register = register;
            this.length = length;
        }
    }
}
